# Generated by Django 4.0.3 on 2022-12-15 20:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('whatsapp_business_api_is', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='wauser',
            name='disable_bot',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='wauser',
            name='failure_count',
            field=models.IntegerField(default=0),
        ),
    ]
